#include "particle.h"
#include <ctime>
#include <cmath>
#define PI 3.141592
PartikelTest::PartikelTest(QWidget *parent)
   : QWidget(parent)
{
		//update every 42ms ~ 24 FPS 
		updateTime = 42;
		// window Size, can be bigger or smaller. small = more performance
		winSize = 100;
		
		// start hooking on windows
		ht = new HookThread;
		ht->setDpi(this->logicalDpiX());
		ht->start(QThread::IdlePriority);
		
		// some window specific options (transparent, click through, no borders etc)
		resize((int)winSize,(int)winSize);
		setAttribute(Qt::WA_NoSystemBackground);
		setAttribute(Qt::WA_OpaquePaintEvent);
		setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint | Qt::SubWindow);
		#ifdef WIN32
		SetWindowLong(winId(), 
              GWL_EXSTYLE, 
              GetWindowLong(winId(), GWL_EXSTYLE) | WS_EX_LAYERED |WS_EX_TRANSPARENT); 
		#endif
		
		srand(time(NULL));
		
		
		initParticleSystem();
		
		// last the connections
		updateTimer = new QBasicTimer();
		updateTimer->start(updateTime, this);
		connect(ht, SIGNAL(mouseHooked()), this, SLOT(getMousePos()));
		
		// and the trayicon
		createTrayIcon();
}

void PartikelTest::initParticleSystem()
{
	// init forces, umber of particles, max position, min position etc
	TotForce.y=0;
	TotForce.x=0;
	int n_forces=1;
	Vector2d f[1];
	f[0].x=0.0;
	f[0].y=0.2;
	Vector2d vel,pos1,pos2;
	vel.x=3;
	vel.y=2;
	pos1.x=width()/2;
	pos1.y=height()/2;
	pos2.x=width()/2;
	pos2.y=height()/2;
	Init(250,n_forces,f,Create,vel,pos1,pos2,25,QColor(255,0,0,180),4);
}
		
void PartikelTest::timerEvent(QTimerEvent *e)
{
	if(e->timerId() == updateTimer->timerId())
	{
		// update every 
		upTimer = true;
		update();
	}
}
void PartikelTest::createTrayIcon()
{
	trayIcon = new QSystemTrayIcon(QIcon(":/images/mouse.png"),this);
	
	actionBeenden = new QAction("Beenden",this);
	connect(actionBeenden, SIGNAL(triggered()), qApp, SLOT(quit()));
	
	trayMenu = new QMenu("MausOMeter");
	trayMenu->addAction(actionBeenden);
	
	trayIcon->setContextMenu(trayMenu);
	trayIcon->show();
}
void PartikelTest::getMousePos()
{
	mousePos.x = ht->getMousePos().toPoint().x();
	mousePos.y = ht->getMousePos().toPoint().y();
	QPointF p = ht->getMousePos();
	QPointF po = ht->getOldMousePos();
	pixelsMovedX = p.x()- po.x();
	pixelsMovedY = p.y()- po.y();

	//if(x()+winSize/2 > QCursor::pos().x()+2 && x()+winSize/2 < QCursor::pos().x()-2
	//   && y()+winSize/2 > QCursor::pos().y()+2 && y()+winSize/2 < QCursor::pos().y()-2)
	//	move((int)(mousePos.x-width()/2), (int)(mousePos.y-width()/2));
	if(ht->getLeftClicks() > 0 || ht->getRightClicks() > 0)
	{
		click = true;
		ht->setLeftClicks(0);
		ht->setRightClicks(0);
	}
	if(!click)
	{
		upTimer = true;
		for(int i=0;i<ParticleNum;i++)
		{
			    
			//Particles[i].Pos.x+=pixelsMovedX;
			//Particles[i].Pos.y+= pixelsMovedY;
			//Particles[i].Vel.x+=pixelsMovedX;
			//Particles[i].Vel.y+= pixelsMovedY;
			
			double xy0 = winSize/2;
			double A = pow(xy0-Size*2,2);
			double Axy = pow(Particles[i].Pos.x-xy0,2) + pow(Particles[i].Pos.y-xy0,2);
		    if( Axy > A) 
			{
				Particles[i].Pos.x-=Particles[i].Vel.x;
				Particles[i].Pos.y-=Particles[i].Vel.y;
				double alphaX = asin((Particles[i].Pos.x > xy0 ? Particles[i].Pos.x-(xy0-1) : Particles[i].Pos.x)/xy0) * 180.0 / PI;
				double alphaY = asin((Particles[i].Pos.y > xy0 ? Particles[i].Pos.y-(xy0-1) : Particles[i].Pos.y)/xy0) * 180.0 / PI;
				Particles[i].Vel.y*=-0.5*sin(alphaX);
				Particles[i].Vel.x*=-0.5*sin(alphaY);
			}
				
			
			/*if(Particles[i].Pos.y> (winSize-5) || Particles[i].Pos.y< 5)
			{
				Particles[i].Vel.y*=-0.1;
				//Particles[i].age=Particles[i].LifeSpan;
			}
			if(Particles[i].Pos.x> (winSize-5) || Particles[i].Pos.x< 5) 
			{
				Particles[i].Vel.x*=-0.1;
				//Particles[i].age=Particles[i].LifeSpan;
			}*/
			
		}
		
		//sUpPos->stop();
		//sUpPos->start(50);
		//upPos = true;
		//update();
	}
}
/*void PartikelTest::disableUpPos()
{
	upPos = false;
}*/
void PartikelTest::Init(long num_part,long num_forces, Vector2d Forces[], 
						ParticleType part_type,Vector2d vel, Vector2d pos1,
						Vector2d pos2, int lifespan, QColor color, int size)
{
		Particles = new Particle[num_part]; //num_part is number of particles
		ParticleNum=num_part;
		
	
		for(int i=0;i<num_forces;i++){ //num_forces is number of forces
			
			TotForce.x+=Forces[i].x;
			TotForce.y+=Forces[i].y;
			//qDebug() << QByteArray::number(Forces[i].y);
		}
		Particle_Type=part_type;
		Vel=vel;
		Pos1=pos1;
		Pos2=pos2;
		LifeSpan=lifespan;
		Color=color;
		Size=size;
		
		InitParticles();
}
void PartikelTest::InitParticles()
{
	
	for(int i=0;i<ParticleNum;i++)
		InitParticle(i);
}
void PartikelTest::InitParticle(int i)
{
	Particles[i].Pos.x = rand() % (int)(Pos2.x-Pos1.x+1)+winSize/2; //Random X position inside the rectangle
	Particles[i].Pos.y = rand() % (int)(Pos2.y-Pos1.y-1)+winSize/2; //Random Y position inside the rectangle
	Particles[i].Vel.x = (rand() % (100+1)/100.0) * Vel.x - Vel.x/2; //Random velocity
	//if(i == 0)
	//	qDebug() << QByteArray::number((rand() % (100+1)/100.0) * Vel.y - Vel.y/2);
	Particles[i].Vel.y = (rand() % (100+1)/100.0) * Vel.y - Vel.y/2; //Random velocity
	Particles[i].color=Color;
	Particles[i].size=Size;
	Particles[i].age=0; //New born baby
	Particles[i].LifeSpan=rand() % (int)(LifeSpan)+1; //Random lifespan
	
}
void PartikelTest::paintEvent(QPaintEvent *e)
{
	if(upTimer)
	{
		QPixmap part = drawParticleSyste();
		if(click)
			click = false;
		
		#ifdef WIN32
		updateAlpha(part);
		#else
		setMask(part.mask());
		QPainter p;
		p.begin(this);
		p.fillRect(rect(), Qt::black);
		p.drawPixmap(0,0,part);
		p.end();
		#endif
		
		upTimer = false;
		
		// let the window smooth follow the cursor
		if(x()+winSize/2 > QCursor::pos().x()+2)
			move((int)(x() + (QCursor::pos().x()-(x())-winSize/2)/30),y());
		else if(x()+winSize/2 < QCursor::pos().x()-2)
			move((int)(x() + (QCursor::pos().x()-(x())+winSize/2)/30),y());
		if(y()+winSize/2 > QCursor::pos().y()+2)
			move(x(), (int)(y() + (QCursor::pos().y()-(y())-winSize/2)/30));
		else if(y()+winSize/2 < QCursor::pos().y()-2)
			move(x(), (int)(y() + (QCursor::pos().y()-(y())+winSize/2)/30));
		
		// and let the particle system work
		pixelsMovedY = (QCursor::pos().y()-(y()+winSize/2))/30;
		pixelsMovedX = (QCursor::pos().x()-(x()+winSize/2))/30;
	}
	
}

QPixmap PartikelTest::drawParticleSyste()
{
	// start painting or particles onto a pixmap, a transparent pixmap
	QPixmap part(width(), height());
	part.fill(QColor(0,0,0,0));
	
	QPainter p(&part);
	
	for(int i=0;i<ParticleNum;i++)
	{
		//if particle isnt to old for flying
		if(Particles[i].age >= 0)
		{
			// create a dynamic color for the pixmaps young = nearly white, old = nearly black
		    p.setBrush(Particles[i].color.dark((int)((90/(Particles[i].LifeSpan))*Particles[i].age)+60));
		    p.setPen(Particles[i].color.dark((int)((90/(Particles[i].LifeSpan))*Particles[i].age)+60));
			// draw them
		    p.drawEllipse(QRectF((int)Particles[i].Pos.x,(int)Particles[i].Pos.y,
						Particles[i].size,
						Particles[i].size));
			// now update all variables
			// to the forces add the total force + the pixels the window (and maybe the mouse) moved
		    Particles[i].Vel.x+=TotForce.x-pixelsMovedX/2;
		    Particles[i].Vel.y+=TotForce.y-pixelsMovedY/2;
		    // if a mouse button was clicked let the particles implode and flow around
		    if(click)
		    {
				if(Particles[i].Pos.x>(mousePos.x+width()/2)+10)
					Particles[i].Vel.x*=+3;
				else if(Particles[i].Pos.x<(mousePos.x+width()/2)+10)
					Particles[i].Vel.x*=-3;
				if(Particles[i].Pos.y>(mousePos.y+width()/2)+10)
					Particles[i].Vel.y*=+3;
				else if(Particles[i].Pos.y<(mousePos.y+width()/2)+10)
					Particles[i].Vel.y*=-3;
		    }
			// the new position is the old position + the speed
			Particles[i].Pos.x+=Particles[i].Vel.x;
			Particles[i].Pos.y+=Particles[i].Vel.y;
			
			//if the particles reache a edge, let them bounce
			double xy0 = winSize/2;
			double A = pow(xy0-Size*2,2);
			double Axy = pow(Particles[i].Pos.x-xy0,2) + pow(Particles[i].Pos.y-xy0,2);
		    if( Axy > A) 
			{
				
				Particles[i].Pos.x-=Particles[i].Vel.x;
				Particles[i].Pos.y-=Particles[i].Vel.y;
				double alphaX = asin((Particles[i].Pos.x > xy0 ? Particles[i].Pos.x-(xy0-1) : Particles[i].Pos.x)/xy0) * 180.0 / PI;
				double alphaY = asin((Particles[i].Pos.y > xy0 ? Particles[i].Pos.y-(xy0-1) : Particles[i].Pos.y)/xy0) * 180.0 / PI;
				Particles[i].Vel.y*=-0.5*sin(alphaX);
				Particles[i].Vel.x*=-0.5*sin(alphaY);
				//qDebug() << QByteArray::number(alphaY);
			}
		    //if(Particles[i].Pos.x> || Particles[i].Pos.x< )
			//	Particles[i].Vel.x*=-0.1;
			
			// at least each particle gets older with each update
			Particles[i].age++;
			if(Particles[i].age > Particles[i].LifeSpan)
			{
				if(Particle_Type == Create) 
				{
					InitParticle(i);
				}
				else 
				{
					Particles[i].age = -1;
				} 
			}
		}
	}
	
	p.end();
	return part;
}
#ifdef WIN32

// wont describe this here...
// this function blends a pixmap onto a layered window
void PartikelTest::updateAlpha(QPixmap p)
{
	
	HBITMAP oldBitmap;
	HBITMAP hBitmap;	
    SIZE size;
    size.cx = p.width();
    size.cy = p.height();
    HDC screenDc = GetDC(NULL);
    POINT pointSource;
    pointSource.x = 0;
    pointSource.y = 0; 
    POINT topPos;
    topPos.x = x();
    topPos.y = y();	
    HDC memDc = CreateCompatibleDC(screenDc);
    BLENDFUNCTION blend;
    blend.BlendOp             = AC_SRC_OVER;
    blend.BlendFlags          = 0;
    blend.SourceConstantAlpha = 255;
    blend.AlphaFormat         = AC_SRC_ALPHA;
	hBitmap = p.toWinHBITMAP(QPixmap::PremultipliedAlpha);  // grab a GDI handle from this GDI+ bitmap
	oldBitmap = (HBITMAP)SelectObject(memDc, hBitmap);
	 
	UpdateLayeredWindow(winId(), screenDc,  &topPos,  &size, memDc,  &pointSource, 0, &blend, ULW_ALPHA);
	
	ReleaseDC( NULL, screenDc);
	if (hBitmap != NULL)
	{
		SelectObject(memDc, oldBitmap);
		//DeleteObject(hBitmap); // The documentation says that we have to use the Windows.DeleteObject... but since there is no such method I use the normal DeleteObject from Win32 GDI and it's working fine without any resource leak.
		DeleteObject(hBitmap);
	}
	DeleteDC(memDc); 
}
#else
// under linux we have no hooks, which update our mouse pointer.... we have to do it right here
void PartikelTest::mouseMoveEvent(QMouseEvent *e)
{
	//set old and new position and let us fake a call of the hoothread
	ht->setMousePos(e->globalPos());
	ht->setOldMousePos(mp);
	mp = e->globalPos();
	getMousePos();
}

void PartikelTest::mousePressEvent(QMouseEvent *e)
{
	//fake a click and fake a call of the hoothread
	ht->setLeftClicks(1);
	//ht->setRightClicks(1);
	//mp = e->globalPos();
	getMousePos();
}

#endif
